// -----------------------------------------------------------------------------
// 2024-07-11, klp
// -----------------------------------------------------------------------------
#version 3.8;



// -----------------------------------------------------------------------------
// --- INCLUDES ----------------------------------------------------------------
// -----------------------------------------------------------------------------
#include "colors"
#include "math"


#if (image_width != image_height)
	#error "*** Set image ratio to 1 ! ***"
#end


#declare displayAxis = true;




// -----------------------------------------------------------------------------
// --- SETTINGS ----------------------------------------------------------------
// -----------------------------------------------------------------------------
global_settings {
	assumed_gamma 1.80
	max_trace_level 5 
	}

#default {
	finish {
		ambient 0.00
		emission 0.40
		diffuse 0.60
		}
	} 

// -----------------------------------------------------------------------------
// --- POINTS ------------------------------------------------------------------
// -----------------------------------------------------------------------------


// --- Points ------------------------------------------------------------------
// --- Export from InkScape ----------------------------------------------------
//#declare path35 = prism {
//    linear_sweep
//    bezier_spline
//    1.0, //top
//    0.0, //bottom
//    12 //nr points
//    /*   0*/ <249.99999874, 886.42857827>, <316.34878488, 854.35384819>, <403.57141417, 756.42857575>, <312.85712126, 640.71428031>,
//    /*   1*/ <312.85712126, 640.71428031>, <222.14285480, 525.00000000>, <128.57142803, 566.42857323>, <109.28570835, 735.00000000>,
//    /*   2*/ <109.28570835, 735.00000000>, <90.00000000, 903.57142677>, <183.65121260, 918.50330457>, <249.99999874, 886.42857827>
//	}
//

#declare n = 12;
#declare a = array[n] {
	<249.99999874, 886.42857827>, // pt - red square
	<316.34878488, 854.35384819>,
	<403.57141417, 756.42857575>,
	<312.85712126, 640.71428031>, 
   <312.85712126, 640.71428031>, // pt - green square
   <222.14285480, 525.00000000>,
   <128.57142803, 566.42857323>,
   <109.28570835, 735.00000000>,
   <109.28570835, 735.00000000>, // pt - blue square
   < 90.00000000, 903.57142677>,
   <183.65121260, 918.50330457>,
   <249.99999874, 886.42857827>
	}



// --- Find minimum values -----------------------------------------------------
#declare xmin = +1e8;
#declare ymin = +1e8;

#declare i = 0;
#while ( i < n)
	#declare p = a[i];
	#declare px = p.x;
	#declare xmin = min(px,xmin);
	#declare py = p.y;
	#declare ymin = min(py,ymin);
	#declare i = i + 1;
#end

#debug "\n\n"
#debug concat("x min = ",str(xmin,0,-1),", y min = ",str(ymin,0,-1),"\n")

// --- Translate points --------------------------------------------------------
#declare i = 0;
#while ( i < n)
	#declare a[i] = <a[i].x - xmin, a[i].y - ymin>;
	#declare i = i + 1;
#end

// --- Find minimum and maximum ------------------------------------------------
#declare xmin = +1e8;
#declare xmax = -xmin;

#declare ymin = +1e8;
#declare ymax = -ymin;

#declare i = 0;
#while ( i < n)
	#declare p = a[i];
	
	#declare px = p.x;
	#declare xmin = min(px,xmin);
	#declare xmax = max(px,xmax);

	#declare py = p.y;
	#declare ymin = min(py,ymin);
	#declare ymax = max(py,ymax);

	#declare i = i + 1;
#end

#debug "\n\n"
#debug concat("x min = ",str(xmin,0,-1),", x max = ",str(xmax,0,-1),"\n")
#debug concat("y min = ",str(ymin,0,-1),", y max = ",str(ymax,0,-1),"\n")
#declare MAX = max(xmax,ymax)*0.5;





// -----------------------------------------------------------------------------
// --- SCENE -------------------------------------------------------------------
// -----------------------------------------------------------------------------
#declare CameraAngle = 36;
#declare CameraDistance = 1.25*MAX/tand(CameraAngle*0.50);
#debug concat("Camera distance = ",str(CameraDistance,0,0),"\n")
camera {
	orthographic
	location <MAX, MAX, -CameraDistance>
	up y
	right x*image_width/image_height
	look_at <MAX, MAX, 0>
	angle CameraAngle
	}

light_source {
	<MAX, MAX, -10000>
	color White
	}

background {
	color White*0
	}

plane {
	z,0
	pigment { checker White,White*0.96 scale 50}
	}

#if (displayAxis)
	#declare ArrowLen = 10;
	#declare AxisR = 0.50;

	#declare FontName="times-it";
	#declare FontScale = 20;

	#declare XStr = text { ttf FontName "x" 0.25 <0, 0, 0> scale FontScale }
	#declare YStr = text { ttf FontName "y" 0.25 <0, 0, 0> scale FontScale }

	union {
		cylinder { <-25,0,0>, <MAX*2,0,0>, AxisR }
		cylinder { <MAX*2,0,0>, <MAX*2-ArrowLen,+ArrowLen*tand(20),0> AxisR*1.50 }
		cylinder { <MAX*2,0,0>, <MAX*2-ArrowLen,-ArrowLen*tand(20),0> AxisR*1.50 }
		
		
		cylinder { <0, -25, 0>, <0, +MAX*2, 0>, AxisR }
		cylinder { <0, +MAX*2, 0>, <-ArrowLen*tand(20), MAX*2-ArrowLen, 0>, AxisR*1.50 }
		cylinder { <0, +MAX*2, 0>, <+ArrowLen*tand(20), MAX*2-ArrowLen, 0>, AxisR*1.50 }

		object { XStr translate <MAX*2, 5, -5> }
		object { YStr translate <-18, MAX*2, -5> }
		
		pigment { color Black }
		}
#end



// -----------------------------------------------------------------------------
// --- FUNCTIONS ---------------------------------------------------------------
// -----------------------------------------------------------------------------

#declare fnLine = function (x,_a,_b) { _a*x+_b }
#declare mod2 = function (x,y) { select (mod(x,y), mod(x,y)+y, mod(x,y) ) }



// -----------------------------------------------------------------------------
// --- MACROS ------------------------------------------------------------------
// -----------------------------------------------------------------------------
#macro DrawSquare(p,r,c)
	box {
		<p.x-r*0.50, p.y-r*0.50, 0.0>, <p.x+r*0.50, p.y+r*0.50, -1>
		pigment { color c }
		}
#end

#macro DrawPoint(p,r,c)
	cylinder {
		<p.x, p.y, 0.0>, <p.x, p.y, -1>, r
		pigment { color c }
		}
#end



#declare rot_x = function(x,y,ad) { x*cosd(ad)-y*sind(ad) }
#declare rot_y = function(x,y,ad) { x*sind(ad)+y*cosd(ad) }

#macro RotateZ(Point, Angle)
	<rot_x(Point.x,Point.y,Angle), rot_y(Point.x,Point.y,Angle)>
#end




// -----------------------------------------------------------------------------
// --- DRAWING... ---------------------------------------------------------------
// -----------------------------------------------------------------------------


// --- Draw points and handles -------------------------------------------------
#declare Colors = array[3] { Red, Green, Blue };
#declare C = 0;

#declare i = 0;
#while ( i < n )
	// --- point
	#declare p0 = a[i];
	DrawSquare(p0, 6, Colors[C])
	
	// control point 1
	#declare p01 = a[i+1];
	DrawPoint(p01, 2, Colors[C])

	// control point 2
	#declare p02 = a[i+2];
	DrawPoint(p02, 2, Colors[C])

	// --- two handles
	cylinder{ p0, p01, 0.50 pigment { Colors[C] } }
	cylinder{ p0, p02, 0.50*2 pigment { Colors[C] } }

	#declare C = C + 1;
	#declare i = i + 4;
#end



// --- Draw spline -------------------------------------------------------------
prism {
	linear_sweep
	bezier_spline
	0.0, 0.1,
	n
	#declare i = 0;
	#while( i < n )
		a[i],
		#declare i = i + 1;
	#end
	rotate -90*x
	pigment { color Orange }
	}
	
	

// --- Tangents = handles ? ----------------------------------------------------

#macro LineEq(p1, p2)
	#local _a = (p1.y - p2.y)/(p1.x - p2.x);
	#local _b = p1.y - _a*p1.x;
	(_a,_b)
#end


#declare ArrowX = 8;
#declare ArrowY = ArrowX*tand(20);
#declare ArrowR = 0.4;

#declare i = 0;
#while ( i < n)

	#declare p0 = a[i];
	#declare p1 = a[i+1];
	#declare (p,q) = LineEq(p0,p1);

	#declare _x1 = -0.10*50;
	#declare _y1 = fnLine(_x1,p,q);
	#declare _x2 = 6.5*50;
	#declare _y2 = fnLine(_x2,p,q);
	// tangent to the curve at p0
	cylinder { <_x1,_y1,0>,<_x2,_y2,0>,0.2 pigment { color Black } }

	#declare dy = p1.y-p0.y;
	#declare dx = p1.x-p0.x;
	#declare Angle = mod2(atan2d(dy,dx),360);
	#debug concat("Angle = ",str(Angle,0,-1),"\n")

	#declare L = 50*1.0;
	#declare A = union {
		// "horizontal" vector
		cylinder { <0,0,0>,<L,0,0>, ArrowR}
		// with arrow
		cylinder { <L,0,0>, <L-ArrowX, +ArrowY,0>, ArrowR*1.25 }
		cylinder { <L,0,0>, <L-ArrowX, -ArrowY,0>, ArrowR*1.25 }
		
		union {
			// "collinear" vector to the tangent
			cylinder { <0,0,0>,<L,0,0>, ArrowR }
			cylinder { <L,0,0>, <L-ArrowX, +ArrowY,0>, ArrowR*1.25 }
			cylinder { <L,0,0>, <L-ArrowX, -ArrowY,0>, ArrowR*1.25 }
			rotate Angle*z
			}
		}
	object { A translate p0 pigment { color Cyan } }
	
	// rotated vectors
	object { A rotate 180*z translate p0 pigment { color Magenta }}
	
	#declare i = i + 4;
#end





	
// --- eof ---------------------------------------------------------------------






